function loadApp(data) {
  return new Vue({
    el: '#app',
    data: data,
    methods: {
      bench_publish: function(server1,server2,name) {
        this.started_publish_at = (new Date()).getTime();
        this.started_played_at = null;
        this.pub_webrtc(server1,name);
        this.play(server2,name);
        // this.monitor_timer = setInterval(function() {

        // }, 500);
      },

      pub_webrtc: function(server,name) {
        var video = document.getElementById("webrtc");
        this.webrtcPublisher = webrtcPublish({
          url: "wss://"+window.location.hostname+":"+server.config.https[0]+"/"+name+"/webrtc/publish",
          localVideo: video
        });
        this.webrtc_publishing = true;
      },

      unpub_webrtc: function(server) {
        this.webrtcPublisher.stop();
        this.webrtc_publishing = false;
      },

      pause: function(server) {
        fetch("https://"+window.location.hostname+":"+server.config.https[0]+"/flussonic/api/modify_config",
          {
            method: "post", credentials: 'same-origin',
            headers: new Headers({ "Authorization": "Basic Zmx1c3NvbmljOnBhc3M=" }),
            body: JSON.stringify({
              streams: {
                "transcode1": {
                  urls: [
                    {url: "file://vod/stub_av.mp4"},
                    {url: "m4s://"+server.name+".local:"+server.config.http[0]+"/live1/m4s?prepush=false"},
                  ]
                }
              }
            })
          })
        .then(function(response) { return response.json() });
        this.paused = true;
      },

      resume: function(server) {
        fetch("https://"+window.location.hostname+":"+server.config.https[0]+"/flussonic/api/modify_config",
          {
            method: "post", credentials: 'same-origin',
            headers: new Headers({ "Authorization": "Basic Zmx1c3NvbmljOnBhc3M=" }),
            body: JSON.stringify({
              streams: {
                "transcode1": {
                  urls: [
                    {url: "m4s://"+server.name+".local:"+server.config.http[0]+"/live1/m4s?prepush=false"},
                    {url: "file://vod/stub_av.mp4"},
                  ]
                }
              }
            })
          })
        .then(function(response) { return response.json() });
        this.paused = false;
      },

      switch_live: function(server) {
        fetch("https://"+window.location.hostname+":"+server.config.https[0]+"/flussonic/api/modify_config",
          {
            method: "post", credentials: 'same-origin',
            headers: new Headers({ "Authorization": "Basic Zmx1c3NvbmljOnBhc3M=" }),
            body: JSON.stringify({
              streams: {
                "live1": {
                  urls: [
                    {url: "file://vod/stub_av.mp4"},
                    {"output_audio": "aac", url: "publish://"},
                  ]
                }
              }
            })
          })
        .then(function(response) { return response.json() });
        this.webrtc_paused = true;
      },

      restore_live: function(server) {
        fetch("https://"+window.location.hostname+":"+server.config.https[0]+"/flussonic/api/modify_config",
          {
            method: "post", credentials: 'same-origin',
            headers: new Headers({ "Authorization": "Basic Zmx1c3NvbmljOnBhc3M=" }),
            body: JSON.stringify({
              streams: {
                "live1": {
                  urls: [
                    {"output_audio": "aac", url: "publish://"},
                    {url: "file://vod/stub_av.mp4"},
                  ]
                }
              }
            })
          })
        .then(function(response) { return response.json() });
        this.webrtc_paused = false;
      },

      pub_rtmp: function(server) {
        this.rtmp_publishing = true;
        this.command(server.name, 'pub_rtmp', server);
      },

      unpub_rtmp: function(server) {
        this.rtmp_publishing = false;
        this.command(server.name, 'unpub_rtmp', server);
      },

      launch: function(name) {
        fetch("/demo/api/instances/"+name, {method: "post", credentials: 'same-origin'}).then(function(response) {
          return response.json()
        });
      },

      stop: function(name) {
        try {
          window.player1 && window.player1.stop();
        } catch (e) {
          console.error(e);
        }
        this.play_url = "";
        fetch("/demo/api/instances/"+name, {method: "delete", credentials: 'same-origin'}).then(function(response) {
          return response.json()
        });
      },

      command: function(name, cmd, options) {
        options.name = name;
        options.command = cmd;
        options.config.hostname = window.location.hostname;
        fetch("/demo/api/command", {method: "post", credentials: 'same-origin', 
          body: JSON.stringify(options)}).then(function(response) {
          return response.json()
        });
      },

      play: function(server, stream) {
        this.playUrl('wss://'+window.location.hostname+':'+server.config.https[0]+'/'+stream+'/mse_ld');
      },

      playUrl: function(url) {
        var video = document.getElementById('player');
        this.play_url = url;
        this.player_opts = {onProgress: this.onProgress.bind(this)};

        // переименовал в window.player1 потому что window.player === video oO
        // нужно дождаться пока stop произойдет, потом новый инстанс делать
        var stopPromise = window.player1
          ? window.player1.stop()
          : Promise.resolve()

        stopPromise.then(() => {
          window.player1 = new FlussonicMsePlayer(video, url, this.player_opts);
          window.player1.play();
        })
      },

      onProgress: function(utc) {
        if(this.started_publish_at && !this.started_played_at) {
          this.started_played_at = (new Date()).getTime();
          console.log("play after ms:",this.started_played_at - this.started_publish_at);
        }
        this.play_delay = Math.floor((new Date()).getTime() / 1000 - utc);
      },

      seek: function() {
        var offset = document.getElementById('seek').value;
        if (offset == "0" || offset == "") {
          window.player1.seek("live");
        } else if (parseInt(offset, 10) == offset) {
          var now = Math.floor((new Date()).getTime() / 1000);
          window.player1.seek(now - Math.abs(offset));
        } else {
          console.error("invalid offset!", offset);
        }
      }
    }
  });
}


function webrtcPublish(options) {
  var websocket;
  var peerConnection;
  var video = options.localVideo;
  function sendWebSocketMessage(data) {
    websocket.send(JSON.stringify(data));
  }
  function onWebSocketMessage(event) {
    var message = JSON.parse(event.data);
    switch (message.type) {
      case "answer":
        var description = new window.RTCSessionDescription(message);
        peerConnection.setRemoteDescription(description);
      break;
      case "candidate":
        var candidate = new window.RTCIceCandidate(message.candidate);
        peerConnection.addIceCandidate(candidate);
      break;
    }
  }
  function openWebSocketConnection(url) {
    websocket = new WebSocket(url);
    websocket.onopen = initPeerConnection;
    websocket.onmessage = onWebSocketMessage;
    return {
      stop: stop
    };
  }
  function getMedia() {
    return navigator.mediaDevices.getUserMedia({
      audio: true,
      video: true
    });
  }

  function gotMedia(stream) {
    if(video) {
      video.srcObject = stream;
    }
    peerConnection.addStream(stream);
    
    peerConnection.createOffer({
      "offerToReceiveAudio": true,
      "offerToReceiveVideo":true
    }).then(function(description) {
      return peerConnection.setLocalDescription(description);
    }).then(function() {
      sendWebSocketMessage(peerConnection.localDescription)
    })
  }
  function initPeerConnection() {
    peerConnection = new window.RTCPeerConnection(null);
    peerConnection.stream_id = "local1";
    peerConnection.onicecandidate = gotIceCandidate;
    getMedia().then(gotMedia);
  }
  function gotIceCandidate(event){
    var candidate = event.candidate;
    if (candidate) {
      sendWebSocketMessage({
        type: 'candidate',
        stream_id : "local1",
        label: candidate.sdpMLineIndex,
        id: candidate.sdpMid,
        candidate: candidate
      });
    }
  }
  function stop() {
    peerConnection && peerConnection.close();
    peerConnection = null;
    if (video && video.srcObject) {
      video.srcObject.getTracks().forEach(function(track, idx, tracks) {
        track.stop();
      });
    }
    websocket && websocket.close();

  }
  return openWebSocketConnection(options.url);
}
