const {
  Player,
  Publisher,
  PUBLISHER_EVENTS,
  PLAYER_EVENTS,
  PUBLISHER_STATUSES,
} = window.FlussonicWebRTC;

function loadApp(data) {
  return new Vue({
    el: "#app",
    data: data,
    wrtcPlayer: null,                                                                                                                                                                         
    wrtcPublisher: null,
    methods: {
      pub_webrtc: function () {
        if(data.stream == ""){
            console.log("empty stream name!");
            return;
        }
        data.stream = data.stream.split(' ').join('');
        console.log(data.stream);
        var that = this;
        var url = this.https() + "/dynamic-mosaic/"+data.room;
        var json = { command: "add_stream" };
        json.args = [data.stream];
        fetch(url, {
          method: "post",
          credentials: "same-origin",
          headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
          body: JSON.stringify(json),
        })
          .then(function (response) {
            this.restart && clearTimeout(this.restart);
            that.publish(
              "https://" + data.host + ":" + data.port + "/" + data.stream
            );
            response.json().then(function (json) {
              console.log(json);
              that.track_id = json.track_id;
            });
            this.stream = data.stream;
          })
          .catch(() => {
            this.restart = setTimeout(() => {
              this.pub_webrtc();
            }, 5000);
          });

        data.media = true;
      },

      mute_video: function () {
        data.vmute = !data.vmute;
        var url = this.https() + "/dynamic-mosaic/"+data.room;
        var json = { command: "mute_video" };
        json.args = [data.stream, data.vmute];
        fetch(url, {
          method: "post",
          credentials: "same-origin",
          headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
          body: JSON.stringify(json),
        }).then(function (response) {
          return response.json().then(function(json){console.log(json);});
        });
      },

      mute_audio: function () {
        data.amute = !data.amute;
        // var url = this.https() + "/dynamic-mosaic/chat";
        // var json = { command: "mute_audio" };
        // json.args = [data.stream, data.amute];
        // fetch(url, {
        //   method: "post",
        //   credentials: "same-origin",
        //   headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
        //   body: JSON.stringify(json),
        // }).then(function (response) {
        //   return response.json().then(function(json){console.log(json);});
        // });
        if (this.wrtcPublisher && this.wrtcPublisher.mute) {
          this.wrtcPublisher.mute();
        }
      },

      unpub_webrtc: function () {
        stop();
        this.webrtc_publishing = false;
        var url = this.https() + "/dynamic-mosaic/" + this.room;
        var json = { command: "remove_stream" };
        json.args = [data.stream];
        fetch(url, {
          method: "post",
          credentials: "same-origin",
          headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
          body: JSON.stringify(json),
        }).then(function (response) {
          response.json().then(function(json){console.log(json);});
        });
        data.stream = "";
        data.media = false;
        data.option = [];
        window.location = "/";
      },

      onPlay: function () {
        console.log("Started playing");
      },

      https: function () {
        return "https://" + data.host + ":" + data.port + "/flussonic/api";
      },

      play: function (url) {
        wrtcPlayer = new Player(
          document.getElementById("player"),
          url,
          { retryMax: 10, retryDelay: 1000 },
          true
        );
        const log = (...defaultMessages) => (...passedMessages) =>
          console.log(...[...defaultMessages, ...passedMessages]);
        wrtcPlayer.on(PLAYER_EVENTS.PLAY, () => {
          this.onPlay();
        });
        wrtcPlayer.on(PLAYER_EVENTS.DEBUG, log("Debugging play"));
        wrtcPlayer.play();
      },

      publish: function (url) {
        if (this.wrtcPublisher) this.wrtcPublisher.stop();
        this.wrtcPublisher = new Publisher(
          url,
          { constraints: { video: true, audio: true },
            onWebsocketClose: this.onWebsocketClose,
          },
          true
        );
        this.wrtcPublisher.on(PUBLISHER_EVENTS.STREAMING, () => {
          if (!this.playing) {
            this.playing = true;
            this.play("https://"+data.host+":"+data.port+"/"+data.room+"?filter=tracks:"+data.track_id);
          }
        });
        this.wrtcPublisher.start();
      },

      onWebsocketClose: function (e) {
        console.log("Websocket lost connection", e);
        this.stop();
        this.pub_webrtc();
      },

      stop: function () {
        if (this.wrtcPublisher) {
          this.wrtcPublisher.stop && this.wrtcPublisher.stop();
          this.wrtcPublisher = null;
        }
        if (this.wrtcPublisher) {
          this.wrtcPlayer.destroy && wrtcPlayer.destroy();
          this.wrtcPlayer = null;
        }        
        this.playing = false;
      },

      get_stats: function() {
        var url = this.https() + "/dynamic-mosaic/"+data.room;
        var json = { command: "stat" };
        json.args = [];
        fetch(url, {
          method: "post",
          credentials: "same-origin",
          headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
          body: JSON.stringify(json),
        }).then(function (response) {
          return response.json().then(function(json){data.stats = JSON.stringify(json);});
        });
      },
      get_os_stats: function() {
        var url = this.https() + "/dynamic-mosaic/";
        var json = { command: "stats" };
        json.args = [];
        fetch(url, {
          method: "post",
          credentials: "same-origin",
          headers: new Headers({ Authorization: "Basic Zmx1c3NvbmljOnBhc3M=" }),
          body: JSON.stringify(json),
        }).then(function (response) {
          return response.json().then(function(json){data.os_stats = JSON.stringify(json);});
        });
      },
    }
  });
}
